// Door / Windows detection
// (C)Takero 2016

#define CHILD_LEFT_ID 3
#define CHILD_RIGHT_ID 4
#define CHILD_ID_BATT 2
#define SENSOR_ID 1
#define BUTTON_PIN_LEFT 4
#define BUTTON_PIN_RIGHT 5
#define INTERRUPT 2
#define SLEEP_TIME 0 // sleep forever

// define values for the battery measurement
#define R1 1.0e6
#define R2 470e3
#define VMIN 2.4
#define VMAX 3.0
#define ADC_PRECISION 1023
#define VREF 1.1

// Enable debug prints to serial monitor
#define MY_DEBUG 

// Enable and select radio type attached
#define MY_RADIO_NRF24
#define MY_RF24_PA_LEVEL RF24_PA_LOW
#define MY_BAUD_RATE 115200
#define MY_NODE_ID 30
#define MY_RF24_CHANNEL 105
#define MY_SIGNING_ATSHA204
#define MY_SIGNING_ATSHA204_PIN 17
#define MY_SIGNING_REQUEST_SIGNATURES

#include <SPI.h>
#include <MySensors.h>

char SKETCH_NAME[] = "Door/Window Sensor";
char SKETCH_VERSION[] = "1.0";

int BATTERY_SENSE_PIN = A0;  // select the input pin for the battery sense point

byte oldSwitch_left_State = HIGH; 
byte oldSwitch_right_State = HIGH; 


MyMessage msg_left(CHILD_LEFT_ID,V_TRIPPED);
MyMessage msg_right(CHILD_RIGHT_ID,V_TRIPPED);
MyMessage msgVolt(CHILD_ID_BATT, V_VOLTAGE);

void setup() {
  
  // use the 1.1 V internal reference
  analogReference(INTERNAL);
  pinMode(BUTTON_PIN_LEFT,INPUT);
  digitalWrite(BUTTON_PIN_LEFT,HIGH);
  pinMode(BUTTON_PIN_RIGHT,INPUT);
  digitalWrite(BUTTON_PIN_RIGHT,HIGH);
  // map interrupt 
  PCICR |= (1 << PCIE2);
  PCMSK2 |= (1 << PCINT20)|(1 << PCINT21);
  sei();
}

void presentation() {
  sendSketchInfo(SKETCH_NAME, SKETCH_VERSION);
  present(CHILD_LEFT_ID, S_DOOR);
  present(CHILD_RIGHT_ID, S_DOOR);  
  
}

void loop() {
  
   byte switchState_left = digitalRead (BUTTON_PIN_LEFT);
   byte switchState_right = digitalRead (BUTTON_PIN_RIGHT);
    
  
  if (switchState_left != oldSwitch_left_State) {
     // Send in the new value
     send(msg_left.set(switchState_left==LOW ? 1 : 0));
     Serial.print("Left Tripped");
     oldSwitch_left_State = switchState_left;
  }
    if (switchState_right != oldSwitch_right_State) {
     // Send in the new value
     send(msg_right.set(switchState_right==LOW ? 1 : 0));
     Serial.print("RIGHT Tripped");
     oldSwitch_right_State = switchState_right;
  }

  
  // BATTERY MESSUREMENT ###########################################################################
  // wait for int. reference voltage
  wait(20);
  int inputValue = analogRead(BATTERY_SENSE_PIN);
  
  float voltageDividerFactor = (R1 + R2) / R2;
  float maxValue = voltageDividerFactor * VREF;
  float voltsPerBit = maxValue / ADC_PRECISION;

  float batteryVoltage = voltsPerBit * inputValue;
  Serial.print("Battery Voltage: ");
  Serial.print(batteryVoltage);
  Serial.println(" V");
  send(msgVolt.set(batteryVoltage, 1));
  int batteryPercentage = ((batteryVoltage-VMIN)/(VMAX-VMIN))*100;
  Serial.print("Battery percent: ");
  Serial.print(batteryPercentage);
  Serial.println(" %");
  sendBatteryLevel(batteryPercentage);
  // END BATT #####################################################################################

  sleep(INTERRUPT, CHANGE, SLEEP_TIME);
}

ISR(PCINT2_vect)
{
} 

